#!/usr/bin/python
# 
# Copyright (C) 2009, Intel Corporation.
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms and conditions of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.  
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307 USA.
#
# Authors:
#              	Wei, Zhang <wei.z.zhang@intel.com> 
#              	Hao, Chen <hao.h.chen@intel.com>
# condition:
# 1. package names are follow the spec definition exactly
#
# steps:
# 1. rpm/dpkg manager to extract package version(s) by '*<package name>*'
# 2.1. no package. WARNING to install related package
# 2.2.1. package(s) found. [packagename]=[required] and >=[required version], PASS
# 2.2.2. package(s) found. [packagename]=[required] and <[required version], WARNING to check the output manually
# 2.2.3. package(s) found. [packagename]<>[required], WARNING to check the output manually

import os
import sys
import re

sys.path.append('../../tetj/')
from tetj import *

sys.path.append('../utils/')
from shell_exec import *

M2CT_PKGS_FILE = 'data/M2CT_PKGS'

# Parse package version
def __parse_pkg_version(pkg_ver_str):

    pkg_ver_str = "\n" + pkg_ver_str
    res = re.findall(r'[-:\s]([\d\.-]+\d)', pkg_ver_str)

    if len(res) != 0:
        return res[0]
    else:
        return None

#=======================================
# Get version in system, method 1
def __get_cur_ver_by_rpm(obj):
    ret = shell_exec3("rpm -qa '*%s*' --qf '%%{name}\\t%%{version}\\n'" % obj)

    if 0 != ret[0]:
	return []
    else:
	if ret[1] != "":
	    nvs = ret[1].split('\n')
            return [ [nv.split('\t')[0], __parse_pkg_version(nv.split('\t')[1])] for nv in nvs ]
	else:
	    return []

#=======================================
# Get version in system, method 2
def __get_cur_ver_by_dpkg(obj):
    ret = shell_exec3("dpkg-query -W -f='${Package}\\t${Version}\\n' '*%s*'" % obj)

    if 0 != ret[0]:
	return []
    else:
	if ret[1] != "":
	    nvs = ret[1].split('\n')
            return [ [nv.split('\t')[0], __parse_pkg_version(nv.split('\t')[1])] for nv in nvs ]
	else:
	    return []

# Get package version
def get_pkg_info(pkg_name, is_deb_os):

    if is_deb_os == False:
        # RPM package
        # -----------
	return __get_cur_ver_by_rpm(pkg_name)

    else:
        # Debian package
        # --------------
	return __get_cur_ver_by_dpkg(pkg_name)


# compare version using array-compare 
def compare_pkg_ver(cur_ver, req_ver, is_above_req):
    # not allowed None
    if cur_ver == None or req_ver == None:
        return False

    # deal with '-'
    cur_ver = cur_ver.replace('-', '.')
    req_ver = req_ver.replace('-', '.')

    # split 
    cur_vers = cur_ver.split('.')
    req_vers = req_ver.split('.')
    
    # Convert list members to integer type
    cur_vers_int = [ int(num) for num in cur_vers ]
    req_vers_int = [ int(num) for num in req_vers ]

    # Compare 2 lists
    if is_above_req:
	return cur_vers_int >= req_vers_int
    else:
	return cur_vers_int == req_vers_int

# Test body
def svcpkgchk(journal):

    import csv
    try:
        is_deb_os = False
        if os.path.exists("/etc/debian_version"):
            is_deb_os = True

        print 'Read pkg requirement file: ' + M2CT_PKGS_FILE
        reader = csv.reader(open(M2CT_PKGS_FILE))
        for name, moblin_req_ver, upper_ver in reader:
            name, moblin_req_ver, upper_ver = [ field.strip() for field in (name, moblin_req_ver, upper_ver) ]

	    journal.testcase_start("Package-Version-Checking-" + name)
            journal.purpose_start()
            ret = TETJ_FIP

	    info = ('Checked requirement - Package %s\n' %  name)

            try:
                # Get package version
		info += "[1] get package <%s>\n" %name
                pkg_infos = get_pkg_info(name, is_deb_os)

		# no package found
		if len(pkg_infos) == 0:
		    info += '[WARNING] No package found. Try to install <%s> related package\n' %(name)
		else:
		    info += "[2] compare version for package(s)\n"
		    # Compare package version to M2CT_PKGS requirement
		    cmp_result = False	#True when (True==name_matched && True==compare_pkg_ver)
                    for pkg_info in pkg_infos:
			name_matched = False
			if(pkg_info[0] == name):
			    name_matched = True

                        if compare_pkg_ver(pkg_info[1], moblin_req_ver, (upper_ver == '+')):
		            info += '[pass] package <%s> - current version: <%s>, moblin required version: <%s>%s' \
                                    % (pkg_info[0], pkg_info[1], moblin_req_ver, upper_ver)
			    if True == name_matched:
				cmp_result = True
				ret = TETJ_PASS
		        else:
		            info += '[fail] package <%s> - current version: <%s>, moblin required version: <%s>%s' \
                                    % (pkg_info[0], pkg_info[1], moblin_req_ver, upper_ver)

		    	if True == name_matched:
			    info += " (package name is fully matched)"

			info += "\n"

		    if True == cmp_result:
		    	info += "[PASS] package found with acceptable version\n"
		    if False == cmp_result:
		    	info += "[WARNING] package found for required <%s>, need to check manually from the output above\n" %(name)
            
	    except:
                info += "Exception on testing"

	    # Print final result
	    print info

            journal.testcase_info(0, 0, 0, info)
            journal.result(ret)
            journal.purpose_end()
            journal.testcase_end()

    except (OSError, IOError), (errno, strerror):
        print "error(%s): %s" % (errno, strerror)


if __name__ == '__main__':

    # Check command line parameters, like "-j journal_file"
    if (len(sys.argv) == 3) and (sys.argv[1] == '-j'):
        journal_file = sys.argv[2]
    else:
        sys.stderr.write('Usage: %s -j journal_file\n' % sys.argv[0])
        sys.exit(1)
    
    try:
	journal = Journal(journal_file, 'Moblin Compliance Test - package version check')
    except  IOError:
        sys.stderr.write('Cannot create journal file : %s\n' % journal_file)
        sys.exit(1)

    # svcpkgchk
    svcpkgchk (journal)

    journal.close()
